import hashlib
import hmac
import base64
import re

import argon2
import bcrypt
import zlib
import passlib.hash
import binascii
from enzoic.enums.password_types import PasswordType
from enzoic.exceptions import UnsupportedPasswordType


def base_64_decode(base64_string):
    return base64.b64decode(base64_string + "=" * (-len(base64_string) % 4))


def calc_sha256_unsalted_hash(password: str) -> hex:
    hashed_pw = hashlib.sha256(password.encode("utf-8")).hexdigest()
    return hashed_pw


def calc_sha1_unsalted_hash(password: str) -> hex:
    hashed_pw = hashlib.sha1(password.encode("utf-8")).hexdigest()
    return hashed_pw


def calc_md5_unsalted_hash(password: str) -> hex:
    hashed_pw = hashlib.md5(password.encode("utf-8")).hexdigest()
    return hashed_pw


def calc_ipboard_mybb_hash(password: str, salt: str) -> hex:
    md5_salt = hashlib.md5(salt.encode("utf-8")).hexdigest()
    md5_pw = hashlib.md5(password.encode("utf-8")).hexdigest()
    hashed_pw = hashlib.md5((md5_salt + md5_pw).encode("utf-8")).hexdigest()
    return hashed_pw


def calc_triple_des_hash(password: str, salt: str):
    hashed_pw = passlib.hash.des_crypt.using(salt=salt[:2]).hash(password)
    return hashed_pw


def calc_vbulletin_pre_3_8_5_hash(password: str, salt: str) -> hex:
    hashed_pw = hashlib.md5(
        (hashlib.md5(password.encode("utf-8")).hexdigest() + salt).encode("utf-8")
    ).hexdigest()
    return hashed_pw


def calc_vbulletin_post_3_8_5_hash(password: str, salt: str) -> hex:
    return calc_vbulletin_pre_3_8_5_hash(password, salt)


def calc_bcrypt_hash(password: str, salt: str) -> bcrypt:
    hashed_pw = bcrypt.hashpw(password.encode("utf-8"), salt.encode("utf-8"))
    return hashed_pw.decode()


def calc_crc32_hash(password: str) -> hex:
    hashed_pw = zlib.crc32(password.encode("utf-8"))
    return str(hashed_pw)


def calc_phpbb3_hash(password: str, salt: str) -> hex:
    itoa64 = "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"
    return passlib.hash.phpass.using(
        salt=salt[4:12], rounds=itoa64.index(salt[3]), ident=salt.split("$")[1]
    ).hash(password)


def calc_scrypt_hash(password: str, salt: str):
    """Not currently in use"""
    hashed_pw = passlib.hash.scrypt.using(salt=salt.encode("utf-8")).hash(password)
    return hashed_pw


def calc_custom_algorithm_1_hash(password: str, salt: str) -> hex:
    to_whirlpool = salt + password
    to_sha = password + salt
    # make these byte objects
    sha512_out = hashlib.sha512(to_sha.encode("utf-8")).digest()
    whirlpool_out = hashlib.new("whirlpool")
    whirlpool_out.update(to_whirlpool.encode("utf-8"))
    whirlpool_out = whirlpool_out.digest()
    # xor byte objects
    hashed_pw = bytes(a ^ b for (a, b) in zip(sha512_out, whirlpool_out))
    return hashed_pw.hex()


def calc_custom_algorithm_2_hash(password: str, salt: str) -> hex:
    hashed_pw = hashlib.md5(f"{password}{salt}".encode("utf-8")).hexdigest()
    return hashed_pw


def calc_sha512_unsalted_hash(password: str) -> hex:
    hashed_pw = hashlib.sha512(password.encode("utf-8")).hexdigest()
    return hashed_pw


def calc_custom_algorithm_3_hash(password: str, salt="kikugalanet") -> hex:
    hashed_pw = hashlib.md5(f"{salt}{password}".encode("utf-8")).hexdigest()
    return hashed_pw


def calc_md5crypt_hash(password: str, salt: str) -> passlib.hash.md5_crypt:
    hashed_pw = passlib.hash.md5_crypt.using(salt=salt[:8]).hash(password)
    return hashed_pw


def calc_custom_algorithm_4_hash(password: str, salt: str) -> bcrypt:
    md5_value = hashlib.md5(password.encode("utf-8")).hexdigest()
    hashed_pw = bcrypt.hashpw(md5_value.encode("utf-8"), salt.encode("utf-8"))
    return hashed_pw.decode()


def calc_custom_algorithm_5_hash(password: str, salt: str) -> hex:
    md5_value = hashlib.md5(f"{password}{salt}".encode("utf-8")).hexdigest()
    hashed_pw = hashlib.sha256(md5_value.encode("utf-8")).hexdigest()
    return hashed_pw


def calc_oscommerce_aef_hash(password: str, salt: str) -> hex:
    hashed_pw = hashlib.md5(f"{salt}{password}".encode("utf-8")).hexdigest()
    return hashed_pw


def calc_descrypt_hash(password: str, salt: str):
    hashed_pw = passlib.hash.des_crypt.using(salt=salt[:2]).hash(password)
    return hashed_pw


def calc_mysql_pre_4_1(password: str) -> hex:
    hashed_pw = passlib.hash.mysql323.hash(password)
    return hashed_pw


def calc_mysql_post_4_1(password: str) -> hex:
    hashed_pw = passlib.hash.mysql41.hash(password).lower()
    return hashed_pw


def calc_peoplesoft_hash(password: str) -> hex:
    sha1_hash = hashlib.sha1(password.encode("utf-16-le")).digest()
    hashed_pw = base64.b64encode(sha1_hash).decode()
    return hashed_pw


def calc_punbb_hash(password: str, salt: str) -> hex:
    sha1_hash = hashlib.sha1(password.encode("utf-8")).hexdigest()
    hashed_pw = hashlib.sha1(f"{salt}{sha1_hash}".encode("utf-8")).hexdigest()
    return hashed_pw


def calc_sha1_salted_hash(password: str, salt: str) -> hex:
    hashed_pw = hashlib.sha1(f"{password}{salt}".encode("utf-8")).hexdigest()
    return hashed_pw


def calc_partial_md5_20_hash(password: str) -> hex:
    hashed_pw = calc_md5_unsalted_hash(password)[:20]
    return hashed_pw


def calc_partial_md5_29_hash(password: str) -> hex:
    hashed_pw = calc_md5_unsalted_hash(password)[:29]
    return hashed_pw


def calc_ave_datalife_diferior_hash(password: str) -> hex:
    hashed_pw = calc_md5_unsalted_hash(
        calc_md5_unsalted_hash(password)
    )
    return hashed_pw


def calc_django_md5_hash(password: str, salt: str) -> hex:
    hashed_pw = f'md5${salt}${calc_md5_unsalted_hash(f"{salt}{password}")}'
    return hashed_pw


def calc_django_sha1_hash(password: str, salt: str) -> hex:
    hashed_pw = (
        f'sha1${salt}${calc_sha1_unsalted_hash(f"{salt}{password}")}'
    )
    return hashed_pw


def calc_pligg_cms_hash(password: str, salt: str) -> hex:
    hashed_pw = f'{salt}{calc_sha1_unsalted_hash(f"{salt}{password}")}'
    return hashed_pw


def calc_run_cms_smf1_1(password: str, salt: str) -> hex:
    hashed_pw = calc_sha1_unsalted_hash(f"{salt}{password}")
    return hashed_pw


def calc_ntlm_hash(password: str) -> hex:
    hashed_pw = hashlib.new("md4", password.encode("utf-16le")).digest()
    return binascii.hexlify(hashed_pw).decode("utf-8")


def calc_sha1dash_hash(password: str, salt: str) -> hex:
    return calc_sha1_unsalted_hash(f"--{salt}--{password}--")


def calc_sha384_hash(password: str) -> hex:
    return hashlib.sha384(password.encode("utf-8")).hexdigest()


def calc_custom_algorithm_7_hash(password: str, salt: str) -> hex:
    # This key is hardcoded and specific to this algorithm
    byte_key = b"d2e1a4c569e7018cc142e9cce755a964bd9b193d2d31f02d80bb589c959afd7e"
    derived_salt = calc_sha1_unsalted_hash(salt)
    msg = derived_salt.encode("utf-8") + password.encode("utf-8")
    hashed = hmac.new(byte_key, msg, digestmod=hashlib.sha256)
    return hashed.hexdigest()


def calc_custom_algorithm_8_hash(password: str, salt: str) -> hex:
    return calc_sha256_unsalted_hash(salt + password)


def calc_custom_algorithm_9_hash(password: str, salt: str) -> hex:
    result = calc_sha512_unsalted_hash(password + salt)
    for _ in range(11):
        result = calc_sha512_unsalted_hash(result)
    return result


def calc_sha256crypt_hash(password: str, salt: str) -> hex:
    rounds = 5000
    add_back_in = False
    # extract the number of rounds and the salt value
    if "$rounds=" in salt:
        rounds = int(salt.split("=")[1].split("$")[0])
        salt = re.sub(r"\$rounds=\d{1,}", "", salt)
        add_back_in = True

    result = passlib.hash.sha256_crypt.using(salt=salt[3:], rounds=rounds, relaxed=True).hash(password)
    # if rounds was supplied in the salt and is set to 5000 add it back in as the hashing library spec doesn't
    # require this to be present in the hash and removes it
    if add_back_in and rounds == 5000:
        result = f"{result[:3]}rounds={rounds}${result[3:]}"
    return result


def calc_sha512crypt_hash(password: str, salt: str) -> hex:
    rounds = 5000
    add_back_in = False
    # extract the number of rounds and the salt value
    if "$rounds=" in salt:
        rounds = int(salt.split("=")[1].split("$")[0])
        salt = re.sub(r"\$rounds=\d{1,}", "", salt)
        add_back_in = True

    result = passlib.hash.sha512_crypt.using(salt=salt[3:], rounds=rounds, relaxed=True).hash(password)
    # if rounds was supplied in the salt and is set to 5000 add it back in as the hashing library spec doesn't
    # require this to be present in the hash and removes it
    if add_back_in and rounds == 5000:
        result = f"{result[:3]}rounds={rounds}${result[3:]}"
    return result


def calc_custom_algorithm_10_hash(password: str, salt: str) -> hex:
    return calc_sha512_unsalted_hash(password + ":" + salt)


def calc_hmac_sha1_salt_as_key(password: str, salt: str) -> hex:
    hashed = hmac.new(salt.encode("utf-8"), password.encode("utf-8"), hashlib.sha1)
    return hashed.hexdigest()


def calc_auth_me_sha256(password: str, salt: str) -> hex:
    return f"$SHA${salt}${calc_sha256_unsalted_hash(calc_sha256_unsalted_hash(password)+salt)}"


def calc_argon_2_hash(password, salt):
    """Use this for the tests. For calling the api with partial hashes use the calc_argon_2_raw_hash"""
    argon_type = argon2.Type.D
    iterations = 3
    memory_cost = 1024
    parallelism = 2
    hash_length = 20
    just_salt = salt

    # Encode if this comes in as a str
    if isinstance(password, str):
        password = password.encode("utf-8")

    # Check if salt has encoded settings
    if salt.startswith("$argon2"):
        # settings are encoded, use them
        if salt.startswith("$argon2i"):
            argon_type = argon2.Type.I

        salt_components = salt.split("$")
        if len(salt_components) == 5:
            # maker sure b64 encoded salt length is a multiple of 4
            # just_salt = base64.b64decode(salt_components[4] + '=' * (-len(salt_components[4]) % 4)).decode()
            just_salt = base_64_decode(salt_components[4]).decode()
            # loop through and calculate the new params
            salt_params = salt_components[3].split(",")
            for param in salt_params:
                param = param.split("=")
                if param[0] == "t":
                    try:
                        iterations = int(param[1])
                    except:
                        iterations = 3
                elif param[0] == "m":
                    try:
                        memory_cost = int(param[1])
                    except:
                        memory_cost = 1024
                elif param[0] == "p":
                    try:
                        parallelism = int(param[1])
                    except:
                        parallelism = 2
                elif param[0] == "l":
                    try:
                        hash_length = int(param[1])
                    except:
                        hash_length = 20

    # calculate the hash
    argon2_hash = argon2.PasswordHasher(
        time_cost=iterations,
        memory_cost=memory_cost,
        parallelism=parallelism,
        hash_len=hash_length,
        type=argon_type,
    ).hash(password=password, salt=just_salt.encode("utf-8"))
    return argon2_hash


def _calc_credential_hash(
    username, password, argon2_salt, hash_type, password_salt
):
    password_hash = calculate_password_hash(
        password_to_hash=password, salt=password_salt, hash_type=hash_type
    )

    if password_hash is not None:
        argon2_hash = calc_argon_2_hash(
            f"{username}${password_hash}", argon2_salt
        )

        just_hash = argon2_hash.split("$")[-1]

        return base_64_decode(just_hash).hex()
    else:
        return


def calculate_password_hash(hash_type: int, password_to_hash: str, salt=""):
    """Pass in a hashtype int and calculate the required password hash"""
    if hash_type == PasswordType.MD5_UNSALTED:
        return calc_md5_unsalted_hash(password_to_hash)

    elif hash_type == PasswordType.SHA1_UNSALTED:
        return calc_sha1_unsalted_hash(password_to_hash)

    elif hash_type == PasswordType.SHA256_UNSALTED:
        return calc_sha256_unsalted_hash(password_to_hash)

    elif hash_type == PasswordType.TripleDES:
        return calc_triple_des_hash(password_to_hash, salt)

    elif hash_type == PasswordType.IPBoard_MyBB:
        return calc_ipboard_mybb_hash(password_to_hash, salt)

    elif hash_type == PasswordType.VBulletinPre3_8_5:
        return calc_vbulletin_pre_3_8_5_hash(password_to_hash, salt)

    elif hash_type == PasswordType.VBulletinPost3_8_5:
        return calc_vbulletin_post_3_8_5_hash(password_to_hash, salt)

    elif hash_type == PasswordType.BCrypt:
        return calc_bcrypt_hash(password_to_hash, salt)

    elif hash_type == PasswordType.CRC32:
        return calc_crc32_hash(password_to_hash)

    elif hash_type == PasswordType.PHPBB3:
        return calc_phpbb3_hash(password_to_hash, salt)

    elif hash_type == PasswordType.CustomAlgorithm1:
        return calc_custom_algorithm_1_hash(password_to_hash, salt)

    # This one can be ignored as we have no SCRYPT breaches in our database
    # as well as there not being any set standard way to compute it
    elif hash_type == PasswordType.SCrypt:
        return calc_scrypt_hash(password_to_hash, salt)

    elif hash_type == PasswordType.CustomAlgorithm2:
        return calc_custom_algorithm_2_hash(password_to_hash, salt)

    elif hash_type == PasswordType.SHA512:
        return calc_sha512_unsalted_hash(password_to_hash)

    elif hash_type == PasswordType.CustomAlgorithm3:
        return calc_custom_algorithm_3_hash(password_to_hash)

    elif hash_type == PasswordType.MD5Crypt:
        return calc_md5crypt_hash(password_to_hash, salt)

    elif hash_type == PasswordType.CustomAlgorithm4:
        return calc_custom_algorithm_4_hash(password_to_hash, salt)

    elif hash_type == PasswordType.CustomAlgorithm5:
        return calc_custom_algorithm_5_hash(password_to_hash, salt)

    elif hash_type == PasswordType.osCommerce_AEF:
        return calc_oscommerce_aef_hash(password_to_hash, salt)

    elif hash_type == PasswordType.DESCrypt:
        return calc_descrypt_hash(password_to_hash, salt)

    elif hash_type == PasswordType.MySQLPre4_1:
        return calc_mysql_pre_4_1(password_to_hash)

    elif hash_type == PasswordType.MySQLPost4_1:
        return calc_mysql_post_4_1(password_to_hash)

    elif hash_type == PasswordType.PeopleSoft:
        return calc_peoplesoft_hash(password_to_hash)

    elif hash_type == PasswordType.PunBB:
        return calc_punbb_hash(password_to_hash, salt)

    elif hash_type == PasswordType.CustomAlgorithm6:
        return calc_sha1_salted_hash(password_to_hash, salt)

    elif hash_type == PasswordType.PartialMD5_20:
        return calc_partial_md5_20_hash(password_to_hash)

    elif hash_type == PasswordType.AVE_DataLife_Diferior:
        return calc_ave_datalife_diferior_hash(password_to_hash)

    elif hash_type == PasswordType.DjangoMD5:
        return calc_django_md5_hash(password_to_hash, salt)

    elif hash_type == PasswordType.DjangoSHA1:
        return calc_django_sha1_hash(password_to_hash, salt)

    elif hash_type == PasswordType.PartialMD5_29:
        return calc_partial_md5_29_hash(password_to_hash)

    elif hash_type == PasswordType.PliggCMS:
        return calc_pligg_cms_hash(password_to_hash, salt)

    elif hash_type == PasswordType.RunCMS_SMF1_1:
        return calc_run_cms_smf1_1(password_to_hash, salt)

    elif hash_type == PasswordType.NTLM:
        return calc_ntlm_hash(password_to_hash)

    elif hash_type == PasswordType.SHA1Dash:
        return calc_sha1dash_hash(password_to_hash, salt)

    elif hash_type == PasswordType.SHA384:
        return calc_sha384_hash(password_to_hash)

    elif hash_type == PasswordType.CustomAlgorithm7:
        return calc_custom_algorithm_7_hash(password_to_hash, salt)

    elif hash_type == PasswordType.CustomAlgorithm8:
        return calc_custom_algorithm_8_hash(password_to_hash, salt)

    elif hash_type == PasswordType.CustomAlgorithm9:
        return calc_custom_algorithm_9_hash(password_to_hash, salt)

    elif hash_type == PasswordType.SHA512Crypt:
        return calc_sha512crypt_hash(password_to_hash, salt)

    elif hash_type == PasswordType.SHA256Crypt:
        return calc_sha256crypt_hash(password_to_hash, salt)

    elif hash_type == PasswordType.HMACSHA1_SaltAsKey:
        return calc_hmac_sha1_salt_as_key(password_to_hash, salt)

    elif hash_type == PasswordType.AuthMeSHA256:
        return calc_auth_me_sha256(password_to_hash, salt)

    elif hash_type == PasswordType.CustomAlgorithm10:
        return calc_custom_algorithm_10_hash(password_to_hash, salt)

    else:
        raise UnsupportedPasswordType



from enum import IntEnum


class PasswordType(IntEnum):
    PLAINTEXT = 0
    MD5_UNSALTED = 1
    SHA1_UNSALTED = 2
    SHA256_UNSALTED = 3
    TripleDES = 4
    IPBoard_MyBB = 5
    VBulletinPre3_8_5 = 6
    VBulletinPost3_8_5 = 7
    BCrypt = 8
    CRC32 = 9
    PHPBB3 = 10
    CustomAlgorithm1 = 11
    SCrypt = 12
    CustomAlgorithm2 = 13
    SHA512 = 14
    CustomAlgorithm3 = 15
    MD5Crypt = 16
    CustomAlgorithm4 = 17
    CustomAlgorithm5 = 18
    osCommerce_AEF = 19
    DESCrypt = 20
    MySQLPre4_1 = 21
    MySQLPost4_1 = 22
    PeopleSoft = 23
    PunBB = 24
    CustomAlgorithm6 = 25
    PartialMD5_20 = 26
    AVE_DataLife_Diferior = 27
    DjangoMD5 = 28
    DjangoSHA1 = 29
    PartialMD5_29 = 30
    PliggCMS = 31
    RunCMS_SMF1_1 = 32
    NTLM = 33
    SHA1Dash = 34
    SHA384 = 35
    CustomAlgorithm7 = 36
    CustomAlgorithm8 = 37
    CustomAlgorithm9 = 38
    SHA512Crypt = 39
    CustomAlgorithm10 = 40
    HMACSHA1_SaltAsKey = 41
    AuthMeSHA256 = 42
    SHA256Crypt = 43
    Unknown = 97
    UnusablePassword = 98



import pytest
import os
from enzoic import Enzoic
from enzoic import exceptions
from enzoic.enums.password_types import PasswordType


@pytest.fixture
def check_constructor_with_parameters():
    def _check_constructor_with_parameters(api_key: str, secret: str) -> bool:
        try:
            Enzoic(api_key, secret)
        except Exception:
            return True
        return False

    yield _check_constructor_with_parameters


@pytest.fixture(scope="session")
def _get_api_key():
    # Set this variable to run live tests
    return os.environ.get(
        "PP_API_KEY", "There is no environment variable set for PP_API_KEY"
    )


@pytest.fixture(scope="session")
def _get_api_secret():
    # Set this variable to run live tests
    return os.environ.get(
        "PP_API_SECRET", "There is no environment variable set for PP_API_SECRET"
    )


@pytest.fixture
def enzoic(_get_api_key, _get_api_secret):
    def enzoic(api_key=None, api_secret=None, api_base_url="https://api.enzoic.com/v1"):
        if api_key is None:
            api_key = _get_api_key
        if api_secret is None:
            api_secret = _get_api_secret
        return Enzoic(api_key=api_key, api_secret=api_secret, api_base_url=api_base_url)
    yield enzoic


@pytest.fixture(scope="session")
def password_types():
    yield PasswordType


@pytest.fixture(scope="session")
def enzoic_exceptions():
    yield exceptions



import base64
import requests
from enzoic.exceptions import UnexpectedEnzoicAPIError, UnsupportedPasswordType
from enzoic.utilities import hashing
from enzoic.enums.password_types import PasswordType
from urllib.parse import urlencode, quote_plus
from typing import Tuple, Union, Dict
from datetime import datetime


class Enzoic:
    CREDENTIALS_API_PATH = "/credentials"
    PASSWORDS_API_PATH = "/passwords"
    EXPOSURE_API_PATH = "/exposures"
    ACCOUNTS_API_PATH = "/accounts"
    ALERTS_SERVICE_PATH = "/alert-subscriptions"

    def __init__(self, api_key, api_secret, api_base_url="https://api.enzoic.com/v1"):
        """
        Creates a new instance of Enzoic.
        :param api_key: API key provided by Enzoic for your account.
        :param api_secret: API Secret provided by Enzoic for your account.
        :param api_base_url: If you were provided an alternative API endpoint you may pass in
        this additional parameter to overwrite the default value.
        """
        if api_key is None or api_key == "":
            raise ValueError("API Key cannot be null or empty")
        if api_secret is None or api_secret == "":
            raise ValueError("API Secret cannot be null or empty")
        if api_base_url is None or api_base_url == "":
            raise ValueError("API Base URL cannot be null or empty")

        self.api_key = api_key
        self.api_secret = api_secret
        self.api_base_url = api_base_url

        self.auth_string = "basic " + base64.b64encode(
            (api_key + ":" + api_secret).encode("utf-8")
        ).decode("utf-8")

    def __repr__(self):
        return "Enzoic(" + self.api_key + ", " + self.api_secret + ", " + self.api_base_url + ")"

    def check_password(self, password: str) -> bool:
        """
        Checks whether the provided password is in the Enzoic database of known, compromised passwords.
        See: https://www.enzoic.com/docs/passwords-api
        :param password: The plaintext password to be checked
        :return: True if the password is a known, compromised password and should not be used
        """
        md5 = hashing.calc_md5_unsalted_hash(password)
        sha1 = hashing.calc_sha1_unsalted_hash(password)
        sha256 = hashing.calc_sha256_unsalted_hash(password)
        query_string = (
            f"?partial_md5={md5[:10]}&partial_sha1={sha1[:10]}&partial_sha256={sha256[:10]}"
        )
        response = self._make_rest_call(
            self.api_base_url + self.PASSWORDS_API_PATH + query_string, "GET", None
        )

        if response.status_code != 404:
            for candidate in response.json()["candidates"]:
                keys = candidate.keys()
                if (
                    ("md5" in keys and candidate["md5"] == md5)
                    or ("sha1" in keys and candidate["sha1"] == sha1)
                    or ("sha256" in keys and candidate["sha256"] == sha256)
                ):
                    return True
            return False
        else:
            return False

    def check_hashed_password(self, hashed_pw: str, hash_type: int) -> bool:
        """
        Checks whether the provided password is in the Enzoic database of known, compromised passwords. Pass in the type
        of password hash. Supports NTLM, MD5, SHA1, SHA256 (33, 1, 2, 3 respectively). You can utilize the PasswordTypes
        enum for ease of use like so:

        from enzoic.enums.password_types import PasswordType
        PasswordType.NTLM

        See: https://www.enzoic.com/docs/passwords-api
        :param hashed_pw: The full hash of a password you wish to check. Must match the corresponding hash_type
        parameter. Only the first 7 characters will be sent.
        :param hash_type: The int of the respective password hash supplied, possible values are:
         NTLM, MD5, SHA1, SHA256 (33, 1, 2, 3 respectively)
        :return: True if the password is a known, compromised password and should not be used
        """
        if hash_type == PasswordType.NTLM:
            key = "partialNTLM"
        elif hash_type == PasswordType.SHA256_UNSALTED:
            key = "partialSHA256"
        elif hash_type == PasswordType.MD5_UNSALTED:
            key = "partialMD5"
        elif hash_type == PasswordType.SHA1_UNSALTED:
            key = "partialSHA1"
        else:
            raise UnsupportedPasswordType(
                "Unsupported hash type provided. The following values for 'password_type' are supported:"
                f"\nNTLM: {PasswordType.NTLM}"
                f"\nMD5: {PasswordType.MD5_UNSALTED}"
                f"\nSHA1: {PasswordType.SHA1_UNSALTED}"
                f"\nSHA256: {PasswordType.SHA256_UNSALTED}"
            )

        payload = {
            key: hashed_pw[:7]
        }

        response = self._make_rest_call(
            self.api_base_url + self.PASSWORDS_API_PATH, "POST", body=payload
        )

        if response.status_code != 404:
            for candidate in response.json()["candidates"]:
                keys = candidate.keys()
                if (
                    ("md5" in keys and candidate["md5"] == hashed_pw)
                    or ("sha1" in keys and candidate["sha1"] == hashed_pw)
                    or ("sha256" in keys and candidate["sha256"] == hashed_pw)
                    or ("ntlm" in keys and candidate["ntlm"] == hashed_pw)
                ):
                    return True
            return False
        else:
            return False

    def check_password_ex(self, password: str) -> Tuple[bool, bool, None, int]:
        """
        Checks whether the provided password is in the Enzoic database of known, compromised passwords and returns the
        relative exposure frequency as well as exposure Count.
        See: https://www.enzoic.com/docs/passwords-api
        :param password: The plaintext password to be checked
        :returns: True if the password is a known, compromised password and should not be used
        revealed_in_exposure (bool) - If the password was found in a breach or not
        relative_exposure_frequency (bool) - How often we see this password relative to exposures
        exposure_count (int) - the amount of exposures this password has been found in
        """
        revealed_in_exposure = False
        relative_exposure_frequency = None
        exposure_count = 0

        md5 = hashing.calc_md5_unsalted_hash(password)
        sha1 = hashing.calc_sha1_unsalted_hash(password)
        sha256 = hashing.calc_sha256_unsalted_hash(password)
        query_string = (
            f"?partial_md5={md5[:10]}&partial_sha1={sha1[:10]}&partial_sha256={sha256[:10]}"
        )
        response = self._make_rest_call(
            self.api_base_url + self.PASSWORDS_API_PATH + query_string, "GET", None
        )

        if response.status_code != 404:
            for candidate in response.json()["candidates"]:
                keys = candidate.keys()
                if (
                    ("md5" in keys and candidate["md5"] == md5)
                    or ("sha1" in keys and candidate["sha1"] == sha1)
                    or ("sha256" in keys and candidate["sha256"] == sha256)
                ):
                    return (
                        True,
                        candidate["revealedInExposure"],
                        candidate["relativeExposureFrequency"],
                        candidate["exposureCount"],
                    )
            return (
                False,
                revealed_in_exposure,
                relative_exposure_frequency,
                exposure_count,
            )
        else:
            return (
                False,
                revealed_in_exposure,
                relative_exposure_frequency,
                exposure_count,
            )

    def check_credentials(
        self,
        username: str,
        password: str,
        last_check_date: datetime = None,
        exclude_hash_types: list = None
    ) -> bool:
        """
        Calls the Enzoic CheckCredentials API in a secure fashion to check whether the provided username and password
        are known to be compromised. This call is made securely to the server - only a salted and hashed representation
        of the credentials are passed and the salt value is not passed along with it.
        See https://www.enzoic.com/docs/credentials-api

        :param username: The username to check - may be an email address or a username
        :param password: The password to check
        :param last_check_date: (Optional) The timestamp for the last check you performed for this user. If the
        date/time you provide for the last check is greater than the timestamp Enzoic has for the last breach affecting
        this user, the check will not be performed. This can be used to substantially increase performance. Can be set
        to None if no last check was performed or the credentials have changed since.
        :param exclude_hash_types: (Optional) An array of PasswordTypes to ignore when calculating hashes for the
        credentials check. By excluding computationally expensive PasswordTypes, such as BCrypt, it is possible to
        balance the performance of this call against security. Can be set to null if you don't wish to exclude any hash
        types.
        :return: True if the credentials are known to be compromised, otherwise False.
        """
        # username needs to be converted to lowercase and url encoded
        params = {"username": str(username).lower()}
        result = urlencode(params, quote_via=quote_plus)

        response = self._make_rest_call(
            self.api_base_url + self.ACCOUNTS_API_PATH + f"?{result}", "GET", None
        )
        if response.status_code == 404:
            # This is all we needed to check for this, 404 means the email wasn't even in the database
            return False

        # see if the last_check_date was later than the lastBreachDate - if so, bail out
        if (
            last_check_date is not None
            and last_check_date.isoformat() >= response.json()["lastBreachDate"]
        ):
            return False

        bcrypt_count = 0
        credential_hashes = []
        query_string = ""

        for hash_spec in response.json()["passwordHashesRequired"]:
            if (
                exclude_hash_types is not None
                and hash_spec["hashType"] in exclude_hash_types
            ):
                # This type is meant to be excluded
                continue

            # Bcrypt gets far too expensive for good response time if there are many of them to calculate.
            # some (mostly garbage accounts) have accumulated a number of them in our DB and if we happen to hit one
            # it kills performance, so short circuit out after, at most, 2 BCrypt hashes
            if hash_spec["hashType"] != PasswordType.BCrypt or bcrypt_count <= 2:
                if hash_spec["hashType"] == PasswordType.BCrypt:
                    bcrypt_count += 1

                credential_hash = hashing._calc_credential_hash(
                    username=username,
                    password=password,
                    argon2_salt=response.json()["salt"],
                    hash_type=hash_spec["hashType"],
                    password_salt=hash_spec["salt"],
                )
                if credential_hash:
                    credential_hashes.append(credential_hash)
                    if len(query_string) == 0:
                        query_string += f"?partialHashes={credential_hash[:10]}"
                    else:
                        query_string += f"&partialHashes={credential_hash[:10]}"

        if len(query_string) > 0:
            creds_response = self._make_rest_call(
                self.api_base_url + self.CREDENTIALS_API_PATH + query_string,
                "GET",
                None,
            )

            if creds_response.status_code != 404:
                # loop through the candidate hashes returned and see if we have a match with the exact hash
                for candidate in creds_response.json()["candidateHashes"]:
                    if candidate in credential_hashes:
                        return True

        return False

    def get_exposures_for_user(self, username: str) -> Dict:
        """
        Returns all the credentials Exposures that have been found for a given username.
        See: https://www.enzoic.com/docs/exposures-api#get-exposures
        :param username: The username or the email address of the user to check
        :return: The json response contains an array of exposure IDs for the user. These IDs can be used with the
         get_exposure_details call to get additional information about each exposure
        """
        response = self._make_rest_call(
            self.api_base_url + self.EXPOSURE_API_PATH + "?username=" + username,
            "GET",
            None,
        )

        if response.status_code == 404:
            # We don't have this email in the DB - return an empty response
            return {"count": 0, "exposures": []}
        else:
            return response.json()

    def get_exposure_details(self, exposure_id: str) -> Union[Dict, None]:
        """
        Returns the detailed information for a credentials Exposure.
        See: https://www.enzoic.com/docs/exposures-api#get-exposure-details
        :param exposure_id: The ID of the Exposure
        :return: The json response contains the details of the exposure or None if the Exposure ID could not be found.
        """
        response = self._make_rest_call(
            self.api_base_url + self.EXPOSURE_API_PATH + "?id=" + exposure_id,
            "GET",
            None,
        )

        if response.status_code == 404:
            return None
        else:
            return response.json()

    def get_exposed_users_for_domain(
        self,
        domain: str,
        page_size: int = None,
        paging_token: str = None
    ) -> Union[Dict, requests.Response]:
        """
        GetExposedUsersForDomain returns a list of all users for a given email domain who have had credentials revealed
        in exposures. The results of this call are paginated.
        see https://docs.enzoic.com/enzoic-api-developer-documentation/api-reference/exposures-api/get-exposures-for-all-email-addresses-in-a-domain
        :param domain: Name of the domain you wish to check, e.g. enzoic.com
        :param page_size: Can be any value from 1 to 1000. If page_size is not specified, the default is 1000.
        :param paging_token: A value returned with each page of results and should be passed into this call to retrieve
        the next page of results
        :return:
        """
        query_string = "?accountDomain=" + domain

        if page_size:
            query_string += "&pageSize=" + str(page_size)

        if paging_token:
            query_string += "&pagingToken=" + paging_token

        response = self._make_rest_call(
            self.api_base_url + self.EXPOSURE_API_PATH + query_string,
            "GET",
            None,
        )

        if response.status_code == 404:
            # We don't have this email in the DB - return an empty response
            return {"count": 0, "users": []}
        else:
            return response.json()

    def get_exposures_for_domain(
        self,
        domain: str,
        include_exposure_details: bool = False,
        page_size: int = None,
        paging_token: str = None
    ) -> Dict:
        """
        Returns a list of all exposures found involving users with email addresses from a
        given domain with the details for each exposure included inline in the response.
        see https://docs.enzoic.com/enzoic-api-developer-documentation/api-reference/exposures-api/get-exposures-for-a-domain
        :param domain: The domain to check.
        :param include_exposure_details: If True will include exposure details.
        :param page_size: The results of this call are paginated. page_size can be any value from 1 to 500.
        If page_size is not specified, the default is 100.
        :param paging_token: paging_token is a value returned with each page of results and should be passed into this
        call to retrieve the next page of results.
        :return: The result will be an dictionary containing total count and a list of exposure IDs which can be used
        with the get_exposure_details call to retrieve details
        """

        query_string = "?domain=" + domain

        if include_exposure_details:
            query_string += f"&includeExposureDetails={int(include_exposure_details)}"

        if page_size:
            query_string += "&pageSize=" + str(page_size)

        if paging_token:
            query_string += "&pagingToken=" + paging_token

        response = self._make_rest_call(
            self.api_base_url + self.EXPOSURE_API_PATH + query_string,
            "GET",
            None,
        )

        if response.status_code == 404:
            # We don't have this domain in the DB - return empty response
            return {"count": 0, "exposures": []}
        else:
            return response.json()

    def add_user_alert_subscriptions(
        self,
        username_hashes: list,
        custom_data: str = ""
    ) -> Dict:
        """
        Takes an array of email addresses that are added to the list of users your account monitors for new
        credentials exposures.
        see https://docs.enzoic.com/enzoic-api-developer-documentation/api-reference/breach-monitoring-api/breach-monitoring-by-user#add-breach-alert-subscriptions
        :param username_hashes: the array of email addresses that are added to the list of users your account monitors
        :param custom_data: can optionally be used with any string value to tag the new subscription items with a
        custom value.  This value will be sent to your webhook when a new alert is found for one of these users and
        can also be used to lookup or delete entries.
        :return:
        """
        payload = {
            "usernameHashes": [hashing.calc_sha256_unsalted_hash(username_hash.lower()) for username_hash in username_hashes],
        }
        if custom_data != "":
            payload["customData"] = custom_data

        response = self._make_rest_call(
            self.api_base_url + self.ALERTS_SERVICE_PATH, "POST", body=payload,
        )
        return response.json()

    def delete_user_alert_subscriptions(self, username_hashes: list) -> Dict:
        """
        Takes a list of email addresses you wish to remove from monitoring for new credentials exposures.
        see https://docs.enzoic.com/enzoic-api-developer-documentation/api-reference/breach-monitoring-api/breach-monitoring-by-user#remove-breach-alert-subscriptions
        :param username_hashes: The list of email addresses you wish to remove from monitoring.
        :return:
        """
        payload = {
            "usernameHashes": [hashing.calc_sha256_unsalted_hash(username_hash.lower()) for username_hash in username_hashes],
        }
        response = self._make_rest_call(
            self.api_base_url + self.ALERTS_SERVICE_PATH, "DELETE", body=payload,
        )
        return response.json()

    def delete_user_alert_subscriptions_with_custom_data(self, custom_data: str) -> Dict:
        """
        Takes a custom_data value and deletes all alert subscriptions that have that value.
        see https://docs.enzoic.com/enzoic-api-developer-documentation/api-reference/breach-monitoring-api/breach-monitoring-by-user#remove-breach-alert-subscriptions
        :param custom_data: The matching custom data you wish to match alert subscriptions on.
        :return:
        """
        payload = {
            "usernameCustomData": custom_data,
        }
        response = self._make_rest_call(
            self.api_base_url + self.ALERTS_SERVICE_PATH, "DELETE", body=payload,
        )
        return response.json()

    def get_user_alert_subscriptions(
        self,
        page_size: int = None,
        paging_token: str = None
    ) -> Dict:
        """
        This method returns a list of all the users your account is monitoring for new credentials exposures.
        The results of this call are paginated.
        see https://docs.enzoic.com/enzoic-api-developer-documentation/api-reference/breach-monitoring-api/breach-monitoring-by-user#retrieve-current-breach-alert-subscriptions
        :param page_size: Can be any value from 1 to 1000, if page_size is not specified, the default is 1000.
        :param paging_token: Returned with each page of results and should be passed into this call to retrieve the
        next page of results.
        :return:
        """
        return self.get_user_alert_subscriptions_with_custom_data(
            custom_data="",
            page_size=page_size,
            paging_token=paging_token,
        )

    def get_user_alert_subscriptions_with_custom_data(
        self,
        custom_data: str,
        page_size: int = None,
        paging_token: str = None
    ) -> Dict:
        """
        This returns a list of all the users your account is monitoring for new credentials exposures with the provided
        custom_data value. The results of this call are paginated.
        see https://docs.enzoic.com/enzoic-api-developer-documentation/api-reference/breach-monitoring-api/breach-monitoring-by-user#retrieve-current-breach-alert-subscriptions
        :param custom_data:
        :param page_size: Can be any value from 1 to 1000, if page_size is not specified, the default is 1000.
        :param paging_token: Returned with each page of results and should be passed into this call to retrieve the
        next page of results.
        :return:
        """
        query_params = {}

        if custom_data != "":
            query_params["customData"] = custom_data

        if page_size:
            query_params["pageSize"] = str(page_size)

        if paging_token:
            query_params["pagingToken"] = paging_token

        response = self._make_rest_call(
            self.api_base_url + self.ALERTS_SERVICE_PATH + "?" + urlencode(query_params), "GET", None,
        )

        if response.status_code == 404:
            return {"count": 0, "usernameHashes": [], "pagingToken": ""}
        else:
            return response.json()

    def is_user_subscribed_for_alerts(self, username_hash: str) -> bool:
        """
        Takes a username and returns true if the user is subscribed for alerts, false otherwise.
        see https://docs.enzoic.com/enzoic-api-developer-documentation/api-reference/breach-monitoring-api/breach-monitoring-by-user#retrieve-current-breach-alert-subscriptions
        :param username_hash: The username hash you wish to check for alert subscriptions.
        :return:
        """
        username_hash = hashing.calc_sha256_unsalted_hash(username_hash.lower())
        response = self._make_rest_call(
            self.api_base_url + self.ALERTS_SERVICE_PATH + f"usernameHash={username_hash}", "GET", None,
        )

        if response.status_code == 404:
            return False
        else:
            return True

    def add_domain_alert_subscriptions(self, domains: list) -> Dict:
        """
        Takes a list of domains (e.g. enzoic.com) and adds them to the list of domains your account monitors for new
        credentials exposures.
        see https://docs.enzoic.com/enzoic-api-developer-documentation/api-reference/breach-monitoring-api/breach-monitoring-by-domain#add-breach-alert-subscriptions
        :param domains: The list of domains you wish to add to monitoring.
        :return:
        """
        payload = {
            "domains": domains
        }
        response = self._make_rest_call(
            self.api_base_url + self.ALERTS_SERVICE_PATH, "POST", body=payload,
        )
        return response.json()

    def delete_domain_alert_subscriptions(self, domains: list) -> Dict:
        """
        Takes an array of domains you wish to remove from monitoring for new credentials exposures.
        see https://docs.enzoic.com/enzoic-api-developer-documentation/api-reference/breach-monitoring-api/breach-monitoring-by-domain#remove-breach-alert-subscriptions
        :param domains: The list of domains you wish to remove from monitoring.
        :return:
        """
        payload = {
            "domains": domains
        }
        response = self._make_rest_call(
            self.api_base_url + self.ALERTS_SERVICE_PATH, "DELETE", body=payload,
        )
        return response.json()

    def get_domain_alert_subscriptions(self, page_size: int = None, paging_token: str = None) -> Dict:
        """
        Returns a list of all the domains your account is monitoring for new credentials exposures.
        The results of this call are paginated.
        see https://docs.enzoic.com/enzoic-api-developer-documentation/api-reference/breach-monitoring-api/breach-monitoring-by-domain#retrieve-current-breach-alert-subscriptions
        :param page_size: Can be any value from 1 to 1000. If it is not specified, the default is 1000.
        :param paging_token: A value returned with each page of results and should be passed into this call to retrieve
        the next page of results.
        :return:
        """
        query_params = {
             "domains": 1,
        }
        if page_size:
            query_params["pageSize"] = str(page_size)

        if paging_token:
            query_params["pagingToken"] = paging_token

        response = self._make_rest_call(
            self.api_base_url + self.ALERTS_SERVICE_PATH + "?" + urlencode(query_params), "GET", None,
        )
        return response.json()

    def is_domain_subscribed_for_alerts(self, domain: str) -> bool:
        """
        Takes a domain and returns true if the domain is subscribed for alerts, false otherwise.
        see https://docs.enzoic.com/enzoic-api-developer-documentation/api-reference/breach-monitoring-api/breach-monitoring-by-domain#retrieve-current-breach-alert-subscriptions
        :param domain: The domain you wish to check the subscription status of.
        :return:
        """
        response = self._make_rest_call(
            self.api_base_url + self.ALERTS_SERVICE_PATH + f"?domain={domain}", "GET", None,
        )
        if response.status_code == 404:
            return False
        else:
            return True

    def get_user_passwords(self, username: str, include_exposure_details: bool = False) -> Union[bool, Dict]:
        """
        Returns a list of passwords that Enzoic has found for a specific user.  This call must be enabled for your
        account or you will receive a 403 rejection when attempting to call it.
        see https://docs.enzoic.com/enzoic-api-developer-documentation/api-reference/credentials-api/cleartext-credentials-api
        :param username: The username you wish to receive a list of passwords for.
        :param include_exposure_details: Includes the details of the exposure the password was found in if True,
        :return:
        """
        # username needs to be converted to lowercase and url encoded
        query_params = {
            "username": str(username).lower(),
        }

        if include_exposure_details:
            query_params["includeExposureDetails"] = int(include_exposure_details)

        result = urlencode(query_params, quote_via=quote_plus)

        response = self._make_rest_call(
            self.api_base_url + f"/cleartext-credentials?{result}", "GET", None
        )
        if response.status_code == 404:
            return False
        else:
            return response.json()

    def get_user_passwords_by_partial_hash(self, username: str, include_exposure_details: bool = False) -> Union[bool, Dict]:
        """
        Returns a list of passwords that Enzoic has found for a specific user.  This call must be enabled for your
        account or you will receive a 403 rejection when attempting to call it.
        see https://docs.enzoic.com/enzoic-api-developer-documentation/api-reference/credentials-api/cleartext-credentials-api
        :param username: The username you wish to receive a list of passwords for.
        :param include_exposure_details: Includes the details of the exposure the password was found in if True,
        :return:
        """
        # username needs to be converted to sha256 partial hash and url encoded
        query_params = {
            "partialUsernameHash": hashing.calc_sha256_unsalted_hash(str(username).lower())[:8],
        }

        if include_exposure_details:
            query_params["includeExposureDetails"] = int(include_exposure_details)

        result = urlencode(query_params, quote_via=quote_plus)

        response = self._make_rest_call(
            self.api_base_url + f"/cleartext-credentials-by-partial-hash?{result}", "GET", None
        )
        if response.status_code == 404:
            return False
        else:
            return response.json()

    def get_user_passwords_by_domain(self, domain: str, page_size: int = None, paging_token: str = None) -> Union[bool, Dict]:
        """
        See: https://api.enzoic.com/v1/cleartext-credentials-by-domain
        :param domain: The domain you wish to receive a list of exposed users and their passwords for.
        :param page_size: The amount of results returned per request, defaults to 100, max is 500.
        :param paging_token: If there are additional pages of results then use this to get the next page.
        :return:
        """
        query_params = {
            "domain": str(domain).lower(),
        }
        if page_size:
            query_params["pageSize"] = str(page_size)

        if paging_token:
            query_params["pagingToken"] = paging_token

        result = urlencode(query_params, quote_via=quote_plus)

        response = self._make_rest_call(
            self.api_base_url + f"/cleartext-credentials-by-domain?{result}", "GET", None
        )

        if response.status_code == 404:
            return False
        else:
            return response.json()

    def _make_rest_call(
        self, url: str, method: str, body: Dict = None
    ) -> requests.Response:
        headers = {
            "Authorization": self.auth_string,
            "Content-Type": "application/json",
            "Accept": "application/json",
        }
        if method == "POST" or method == "PUT" or method == "DELETE":
            r = requests.request(method, url=url, headers=headers, json=body)
        else:
            r = requests.request(method, url=url, headers=headers)

        if r.status_code not in (200, 201, 404):
            raise UnexpectedEnzoicAPIError(
                f"Unexpected error from Enzoic API: {r.status_code} {r.text}"
            )
        else:
            return r



class UnsupportedPasswordType(Exception):
    def __init__(self, msg="Unsupported Password Type provided."):
        super().__init__(msg)


class UnexpectedEnzoicAPIError(Exception):
    def __init__(self, msg="Unexpected error from Enzoic API"):
        super().__init__(msg)



import os
import pytest
from datetime import datetime


class TestEnzoic:
    def test_constructor_with_null_parameters(self, check_constructor_with_parameters):
        assert check_constructor_with_parameters(None, None) is True

    def test_constructor_with_null_secret(self, check_constructor_with_parameters):
        assert check_constructor_with_parameters("test", None) is True

    def test_constructor_with_null_key(self, check_constructor_with_parameters):
        assert check_constructor_with_parameters(None, "test") is True

    def test_constructor_with_empty_str_parameters(
        self, check_constructor_with_parameters
    ):
        assert check_constructor_with_parameters("", "") is True

    def test_constructor_with_empty_str_key(self, check_constructor_with_parameters):
        assert check_constructor_with_parameters("", "test") is True

    def test_constructor_with_empty_str_secret(self, check_constructor_with_parameters):
        assert check_constructor_with_parameters("test", "") is True

    def test_constructor_with_valid_parameters(self, check_constructor_with_parameters):
        assert check_constructor_with_parameters("test", "test") is False


class TestCheckCredentials:
    def test_email_creds_are_exposed(self, enzoic):
        assert enzoic().check_credentials("test@passwordping.com", "123456") is True

    def test_email_creds_are_not_exposed(self, enzoic):
        assert enzoic().check_credentials("test@passwordping.com", "notvalid") is False

    def test_username_creds_are_exposed(self, enzoic):
        assert enzoic().check_credentials("testpwdpng445", "testpwdpng4452") is True

    def test_username_creds_are_not_exposed(self, enzoic):
        assert enzoic().check_credentials("testpwdpng445", "notvalid") is False

    def test_ignored_hash_type_is_not_exposed(self, enzoic, password_types):
        assert enzoic().check_credentials(
            "testpwdpng445",
            "testpwdpng4452",
            exclude_hash_types=[password_types.VBulletinPost3_8_5],
        ) is False

    def test_last_check_date_on_exposure(self, enzoic):
        assert enzoic().check_credentials(
            "testpwdpng445", "testpwdpng4452", last_check_date=datetime(2018, 3, 1)
        ) is False


class TestGetExposuresForUser:
    def test_no_exposure_for_username(self, enzoic):
        """Test a bad value"""
        response = enzoic().get_exposures_for_user("@@bogus-username@@")
        assert response["count"] == 0
        assert len(response["exposures"]) == 0

    def test_actual_exposure_for_username(self, enzoic):
        response = enzoic().get_exposures_for_user("eicar")
        assert response["count"] == 9
        assert len(response["exposures"]) == 9
        assert set(response["exposures"]) == {
            "5820469ffdb8780510b329cc",
            "58258f5efdb8780be88c2c5d",
            "582a8e51fdb87806acc426ff",
            "583d2f9e1395c81f4cfa3479",
            "59ba1aa369644815dcd8683e",
            "59cae0ce1d75b80e0070957c",
            "5bc64f5f4eb6d894f09eae70",
            "5bdcb0944eb6d8a97cfacdff",
            "653980098502d3ce61f8bfbb"
        }


class TestGetExposureDetails:
    def test_no_exposure_details(self, enzoic):
        """Test a bad exposure ID"""
        assert enzoic().get_exposure_details("111111111111111111111111") is None

    def test_valid_exposure_with_details(self, enzoic):
        """Test a valid exposure ID"""
        response = enzoic().get_exposure_details("5820469ffdb8780510b329cc")
        assert response is not None
        assert "5820469ffdb8780510b329cc" == response["id"]
        assert "last.fm" == response["title"]
        assert "Music" == response["category"]
        assert datetime(2012, 3, 1).isoformat() in response["date"]
        assert "MD5" == response["passwordType"]
        assert {"Emails", "Passwords", "Usernames", "Website Activity"} == set(
            response["exposedData"]
        )
        assert 81967007 == response["entries"]
        assert 1219053 == response["domainsAffected"]


class TestCheckPassword:
    def test_uncompromised_password(self, enzoic):
        assert (
            enzoic().check_password("kjdlkjdlksjdlskjdlskjslkjdslkdjslkdjslkd") is False
        )

    def test_compromised_password(self, enzoic):
        assert enzoic().check_password("123456") is True

    def test_uncompromised_password_no_exposure_stats(self, enzoic):
        (
            compromised,
            revealed_in_exposure,
            relative_exposure_frequency,
            exposure_count,
        ) = enzoic().check_password_ex("kjdlkjdlksjdlskjdlskjslkjdslkdjslkdjslkd")
        assert compromised is False
        assert revealed_in_exposure is False
        assert relative_exposure_frequency is None

    def test_compromised_password_no_exposure_stats(self, enzoic):
        (
            compromised,
            revealed_in_exposure,
            relative_exposure_frequency,
            exposure_count,
        ) = enzoic().check_password_ex("`!(&,<:{`>")
        assert compromised is True
        assert revealed_in_exposure is False
        assert relative_exposure_frequency is None

    def test_compromised_password_has_exposure_stats(self, enzoic):
        (
            compromised,
            revealed_in_exposure,
            relative_exposure_frequency,
            exposure_count,
        ) = enzoic().check_password_ex("password")
        assert compromised is True
        assert revealed_in_exposure is True
        assert relative_exposure_frequency > 0

    def test_hashed_password_by_ntlm(self, enzoic, password_types):
        ntlm_hash = "8846f7eaee8fb117ad06bdd830b7586c"
        assert enzoic().check_hashed_password(hashed_pw=ntlm_hash, hash_type=password_types.NTLM) is True

    def test_hashed_password_by_md5(self, enzoic, password_types):
        md5_hash = "5f4dcc3b5aa765d61d8327deb882cf99"
        assert enzoic().check_hashed_password(hashed_pw=md5_hash, hash_type=password_types.MD5_UNSALTED) is True

    def test_hashed_password_by_sha256(self, enzoic, password_types):
        sha256_hash = "5e884898da28047151d0e56f8dc6292773603d0d6aabbdd62a11ef721d1542d8"
        assert enzoic().check_hashed_password(hashed_pw=sha256_hash, hash_type=password_types.SHA256_UNSALTED) is True

    def test_hashed_password_by_sha1(self, enzoic, password_types):
        sha1_hash = "5baa61e4c9b93f3f0682250b6cf8331b7ee68fd8"
        assert enzoic().check_hashed_password(hashed_pw=sha1_hash, hash_type=password_types.SHA1_UNSALTED) is True

    def test_unsupported_hash_type_provided(self, enzoic, password_types, enzoic_exceptions):
        ntlm_hash = "8846f7eaee8fb117ad06bdd830b7586c"
        with pytest.raises(enzoic_exceptions.UnsupportedPasswordType) as exc_info:
            enzoic().check_hashed_password(hashed_pw=ntlm_hash, hash_type=password_types.VBulletinPost3_8_5)
            assert "Unsupported hash type provided." in str(exc_info.value)
            assert password_types.NTLM in str(exc_info.value)
            assert password_types.SHA1_UNSALTED in str(exc_info.value)
            assert password_types.SHA256_UNSALTED in str(exc_info.value)
            assert password_types.MD5_UNSALTED in str(exc_info.value)

    def test_uncompromised_password_hash(self, enzoic, password_types):
        ntlm_hash = "6708d6fd35e9fbcda86be9703a20af8c6f595a2f"
        assert enzoic().check_hashed_password(hashed_pw=ntlm_hash, hash_type=password_types.NTLM) is False


class TestGetUserPasswords:
    def test_get_user_password(self, enzoic):
        response = enzoic().get_user_passwords("eicar_0@enzoic.com")
        assert len(response["passwords"]) == 4
        assert response["lastBreachDate"] == "2022-10-14T07:02:40.000Z"
        assert response["passwords"] == [
            {
                "hashType": 0,
                "salt": "",
                "password": "password123",
                "exposures": [
                    "634908d2e0513eb0788aa0b9",
                    "634908d06715cc1b5b201a1a"
                ]
            },
            {
                "hashType": 0,
                "salt": "",
                "password": "g0oD_on3",
                "exposures": [
                    "634908d2e0513eb0788aa0b9"
                ]
            },
            {
                "hashType": 0,
                "salt": "",
                "password": "Easy2no",
                "exposures": [
                    "634908d26715cc1b5b201a1d"
                ]
            },
            {
                "hashType": 0,
                "salt": "",
                "password": "123456",
                "exposures": [
                    "63490990e0513eb0788aa0d1",
                    "634908d0e0513eb0788aa0b5"
                ]
            }
        ]

    def test_get_user_password_with_details(self, enzoic):
        response = enzoic().get_user_passwords("eicar_0@enzoic.com", include_exposure_details=True)
        assert len(response["passwords"]) == 4
        assert response["lastBreachDate"] == "2022-10-14T07:02:40.000Z"
        assert response["passwords"] == [
            {
            "hashType": 0,
            "salt": "",
            "password": "password123",
            "exposures": [
                {
                    "category": "Testing Ignore",
                    "date": None,
                    "dateAdded": "2022-10-14T06:59:28.000Z",
                    "domainsAffected": 1,
                    "entries": 5,
                    "exposedData": [
                        "Emails",
                        "Passwords"
                    ],
                    "id": "634908d06715cc1b5b201a1a",
                    "passwordType": "MD5",
                    "source": "Testing - Ignore",
                    "sourceFileCount": 1,
                    "sourceURLs": [],
                    "title": "enzoic test breach 1",
                },
                {
                    "category": "Testing Ignore",
                    "date": None,
                    "dateAdded": "2022-10-14T06:59:30.000Z",
                    "domainsAffected": 1,
                    "entries": 2,
                    "exposedData": [
                        "Emails",
                        "Passwords"
                    ],
                    "id": "634908d2e0513eb0788aa0b9",
                    "passwordType": "Cleartext",
                    "source": "Testing - Ignore",
                    "sourceFileCount": 1,
                    "sourceURLs": [],
                    "title": "enzoic test breach 5"
                }
            ]
        },
        {
            "hashType": 0,
            "salt": "",
            "password": "g0oD_on3",
            "exposures": [
                {
                    "category": "Testing Ignore",
                    "date": None,
                    "dateAdded": "2022-10-14T06:59:30.000Z",
                    "domainsAffected": 1,
                    "entries": 2,
                    "exposedData": [
                        "Emails",
                        "Passwords"
                    ],
                    "id": "634908d2e0513eb0788aa0b9",
                    "passwordType": "Cleartext",
                    "source": "Testing - Ignore",
                    "sourceFileCount": 1,
                    "sourceURLs": [],
                    "title": "enzoic test breach 5"
                }
            ]
        },
        {
            "hashType": 0,
            "salt": "",
            "password": "Easy2no",
            "exposures": [
                {
                    "category": "Testing Ignore",
                    "date": None,
                    "dateAdded": "2022-10-14T06:59:30.000Z",
                    "domainsAffected": 1,
                    "entries": 4,
                    "exposedData": [
                        "Emails",
                        "Passwords"
                    ],
                    "id": "634908d26715cc1b5b201a1d",
                    "passwordType": "MD5",
                    "source": "Testing - Ignore",
                    "sourceFileCount": 1,
                    "sourceURLs": [],
                    "title": "enzoic test breach 4"
                }
            ]
        },
        {
            "hashType": 0,
            "salt": "",
            "password": "123456",
            "exposures": [
                {
                    "category": "Testing Ignore",
                    "date": None,
                    "dateAdded": "2022-10-14T06:59:28.000Z",
                    "domainsAffected": 1,
                    "entries": 5,
                    "exposedData": [
                        "Emails",
                        "Passwords"
                    ],
                    "id": "634908d0e0513eb0788aa0b5",
                    "passwordType": "MD5",
                    "source": "Testing - Ignore",
                    "sourceFileCount": 1,
                    "sourceURLs": [],
                    "title": "enzoic test breach 2"
                },
                {
                    "category": "Testing Ignore",
                    "date": None,
                    "dateAdded": "2022-10-14T07:02:40.000Z",
                    "domainsAffected": 1,
                    "entries": 3,
                    "exposedData": [
                        "Emails",
                        "Passwords"
                    ],
                    "id": "63490990e0513eb0788aa0d1",
                    "passwordType": "Cleartext",
                    "source": "Testing - Ignore",
                    "sourceFileCount": 1,
                    "sourceURLs": [],
                    "title": "enzoic test breach 3",
                }
            ]
        }
    ]

    def test_get_user_password_not_found(self, enzoic):
        response = enzoic().get_user_passwords(username="@@bogus-user@@")
        assert response is False

    def test_account_without_permissions(self, enzoic, enzoic_exceptions):
        with pytest.raises(enzoic_exceptions.UnexpectedEnzoicAPIError) as exc_info:
            enzoic(
                os.environ.get("PP_API_KEY_2"), os.environ.get("PP_API_SECRET_2")
            ).get_user_passwords("eicar_0@enzoic.com")
        assert "Your account is not allowed to make this call.  Please contact sales@enzoic.com." in str(exc_info.value)


class TestGetUserPasswordsByDomain:
    def test_get_passwords_for_domain_default_page_size(self, enzoic):
        domain_response = enzoic().get_user_passwords_by_domain(domain="enzoic.com")
        assert domain_response["count"] == 94
        assert domain_response["pagingToken"] is None
        assert len(domain_response["users"]) == 94

    def test_get_passwords_for_domain_specific_page_size(self, enzoic):
        domain_response = enzoic().get_user_passwords_by_domain(domain="enzoic.com", page_size=10)
        assert domain_response["count"] == 94
        assert domain_response["pagingToken"] is not None
        assert len(domain_response["users"]) == 10

    def test_get_passwords_for_domain_pagination(self, enzoic):
        paged_response = enzoic().get_user_passwords_by_domain(domain="enzoic.com", page_size=10)
        # now get the paginated results
        domain_response = enzoic().get_user_passwords_by_domain(domain="enzoic.com", paging_token=paged_response["pagingToken"])
        assert domain_response["count"] == 94
        assert domain_response["pagingToken"] is None
        assert len(domain_response["users"]) == 84


class TestGetUserPasswordsByPartialHash:
    def test_get_user_password(self, enzoic):
        response = enzoic().get_user_passwords_by_partial_hash("eicar_0@enzoic.com")
        assert response["candidates"] == [
            {
                'usernameHash': '705bce5505615a1437cc005f933f5891c53778ead4b18b44195682b2ae90d7f7',
                'lastBreachDate': '2020-04-23T17:06:16.000Z',
                'passwords': [
                    {'hashType': 0, 'salt': '', 'password': 'sofi130991', 'exposures': ['5825260cfdb8780854c40c84']},
                    {'hashType': 0, 'salt': '', 'password': 'Merda39a2j', 'exposures': ['5820469ffdb8780510b329cc']},
                    {'hashType': 0, 'salt': '', 'password': 'nirvana', 'exposures': ['5ea1cb08d3cef70b4cda265a']}]
            },
            {
                'usernameHash': '705bce557110384a4ce76aa9c33a12af14ac1eee3978ac3076f866aa0d84f07a',
                'lastBreachDate': '2022-10-14T07:02:40.000Z',
                'passwords': [
                    {'hashType': 0, 'salt': '', 'password': 'password123', 'exposures': ['634908d2e0513eb0788aa0b9', '634908d06715cc1b5b201a1a']},
                    {'hashType': 0, 'salt': '', 'password': 'g0oD_on3', 'exposures': ['634908d2e0513eb0788aa0b9']},
                    {'hashType': 0, 'salt': '', 'password': 'Easy2no', 'exposures': ['634908d26715cc1b5b201a1d']},
                    {'hashType': 0, 'salt': '', 'password': '123456', 'exposures': ['63490990e0513eb0788aa0d1', '634908d0e0513eb0788aa0b5']}]
            },
            {
                'usernameHash': '705bce557ebcf8504587e87fb691ae37c2f00655db024f0fc72c29b775533641',
                'lastBreachDate': '2022-05-16T17:20:13.000Z',
                'passwords': [
                    {'hashType': 0, 'salt': '', 'password': '122000888', 'exposures': ['628287cd3e27d0867a43070a']}
                ]
            }
        ]

    def test_get_user_password_with_details(self, enzoic):
        response = enzoic().get_user_passwords_by_partial_hash("eicar_0@enzoic.com", include_exposure_details=True)
        assert response["candidates"] == [
           {
              "usernameHash":"705bce5505615a1437cc005f933f5891c53778ead4b18b44195682b2ae90d7f7",
              "lastBreachDate":"2020-04-23T17:06:16.000Z",
              "passwords":[
                 {
                    "hashType":0,
                    "salt":"",
                    "password":"sofi130991",
                    "exposures":[
                       {
                          "id":"5825260cfdb8780854c40c84",
                          "title":"zoosk.com",
                          "entries":53778792,
                          "date":"2011-12-31T00:00:00.000Z",
                          "category":"Dating",
                          "source":"Unspecified",
                          "passwordType":"MD5",
                          "exposedData":[
                             "Emails",
                             "Passwords",
                             "Usernames"
                          ],
                          "dateAdded":"2016-11-11T01:59:40.000Z",
                          "sourceURLs":[

                          ],
                          "sourceFileCount":1,
                          "domainsAffected":639449
                       }
                    ]
                 },
                 {
                    "hashType":0,
                    "salt":"",
                    "password":"Merda39a2j",
                    "exposures":[
                       {
                          "id":"5820469ffdb8780510b329cc",
                          "title":"last.fm",
                          "entries":81967007,
                          "date":"2012-03-01T00:00:00.000Z",
                          "category":"Music",
                          "source":"Unspecified",
                          "passwordType":"MD5",
                          "exposedData":[
                             "Emails",
                             "Passwords",
                             "Usernames",
                             "Website Activity"
                          ],
                          "dateAdded":"2016-11-07T09:17:19.000Z",
                          "sourceURLs":[

                          ],
                          "sourceFileCount":1,
                          "domainsAffected":1219053
                       }
                    ]
                 },
                 {
                    "hashType":0,
                    "salt":"",
                    "password":"nirvana",
                    "exposures":[
                       {
                          "id":"5ea1cb08d3cef70b4cda265a",
                          "title":"christianpassions.com",
                          "entries":1275759,
                          "date":"2020-02-01T00:00:00.000Z",
                          "category":"Dating",
                          "source":"Unspecified",
                          "passwordType":"Cleartext",
                          "exposedData":[
                             "Passwords",
                             "Usernames"
                          ],
                          "dateAdded":"2020-04-23T17:06:16.000Z",
                          "sourceURLs":[

                          ],
                          "sourceFileCount":1,
                          "domainsAffected":16150
                       }
                    ]
                 }
              ]
           },
           {
              "usernameHash":"705bce557110384a4ce76aa9c33a12af14ac1eee3978ac3076f866aa0d84f07a",
              "lastBreachDate":"2022-10-14T07:02:40.000Z",
              "passwords":[
                 {
                    "hashType":0,
                    "salt":"",
                    "password":"password123",
                    "exposures":[
                       {
                          "id":"634908d06715cc1b5b201a1a",
                          "title":"enzoic test breach 1",
                          "entries":5,
                          "date":None,
                          "category":"Testing Ignore",
                          "source":"Testing - Ignore",
                          "passwordType":"MD5",
                          "exposedData":[
                             "Emails",
                             "Passwords"
                          ],
                          "dateAdded":"2022-10-14T06:59:28.000Z",
                          "sourceURLs":[

                          ],
                          "sourceFileCount":1,
                          "domainsAffected":1
                       },
                       {
                          "id":"634908d2e0513eb0788aa0b9",
                          "title":"enzoic test breach 5",
                          "entries":2,
                          "date":None,
                          "category":"Testing Ignore",
                          "source":"Testing - Ignore",
                          "passwordType":"Cleartext",
                          "exposedData":[
                             "Emails",
                             "Passwords"
                          ],
                          "dateAdded":"2022-10-14T06:59:30.000Z",
                          "sourceURLs":[

                          ],
                          "sourceFileCount":1,
                          "domainsAffected":1
                       }
                    ]
                 },
                 {
                    "hashType":0,
                    "salt":"",
                    "password":"g0oD_on3",
                    "exposures":[
                       {
                          "id":"634908d2e0513eb0788aa0b9",
                          "title":"enzoic test breach 5",
                          "entries":2,
                          "date":None,
                          "category":"Testing Ignore",
                          "source":"Testing - Ignore",
                          "passwordType":"Cleartext",
                          "exposedData":[
                             "Emails",
                             "Passwords"
                          ],
                          "dateAdded":"2022-10-14T06:59:30.000Z",
                          "sourceURLs":[

                          ],
                          "sourceFileCount":1,
                          "domainsAffected":1
                       }
                    ]
                 },
                 {
                    "hashType":0,
                    "salt":"",
                    "password":"Easy2no",
                    "exposures":[
                       {
                          "id":"634908d26715cc1b5b201a1d",
                          "title":"enzoic test breach 4",
                          "entries":4,
                          "date":None,
                          "category":"Testing Ignore",
                          "source":"Testing - Ignore",
                          "passwordType":"MD5",
                          "exposedData":[
                             "Emails",
                             "Passwords"
                          ],
                          "dateAdded":"2022-10-14T06:59:30.000Z",
                          "sourceURLs":[

                          ],
                          "sourceFileCount":1,
                          "domainsAffected":1
                       }
                    ]
                 },
                 {
                    "hashType":0,
                    "salt":"",
                    "password":"123456",
                    "exposures":[
                       {
                          "id":"634908d0e0513eb0788aa0b5",
                          "title":"enzoic test breach 2",
                          "entries":5,
                          "date":None,
                          "category":"Testing Ignore",
                          "source":"Testing - Ignore",
                          "passwordType":"MD5",
                          "exposedData":[
                             "Emails",
                             "Passwords"
                          ],
                          "dateAdded":"2022-10-14T06:59:28.000Z",
                          "sourceURLs":[

                          ],
                          "sourceFileCount":1,
                          "domainsAffected":1
                       },
                       {
                          "id":"63490990e0513eb0788aa0d1",
                          "title":"enzoic test breach 3",
                          "entries":3,
                          "date":None,
                          "category":"Testing Ignore",
                          "source":"Testing - Ignore",
                          "passwordType":"Cleartext",
                          "exposedData":[
                             "Emails",
                             "Passwords"
                          ],
                          "dateAdded":"2022-10-14T07:02:40.000Z",
                          "sourceURLs":[

                          ],
                          "sourceFileCount":1,
                          "domainsAffected":1
                       }
                    ]
                 }
              ]
           },
           {
              "usernameHash":"705bce557ebcf8504587e87fb691ae37c2f00655db024f0fc72c29b775533641",
              "lastBreachDate":"2022-05-16T17:20:13.000Z",
              "passwords":[
                 {
                    "hashType":0,
                    "salt":"",
                    "password":"122000888",
                    "exposures":[
                       {
                          "id":"628287cd3e27d0867a43070a",
                          "title":"readnovel.com",
                          "entries":19125314,
                          "date":"2019-05-01T00:00:00.000Z",
                          "category":"Books",
                          "source":"Unspecified",
                          "passwordType":"MD5",
                          "exposedData":[
                             "Emails",
                             "Passwords",
                             "Usernames",
                             "Phone Numbers",
                             "Genders"
                          ],
                          "dateAdded":"2022-05-16T17:20:13.000Z",
                          "sourceURLs":[

                          ],
                          "sourceFileCount":1,
                          "domainsAffected":297923
                       }
                    ]
                 }
              ]
           }
        ]

    def test_get_user_password_not_found(self, enzoic):
        response = enzoic().get_user_passwords_by_partial_hash(username="!!!!!@@bogus-user@@!!!!!")
        assert len(response["candidates"]) == 0

    def test_account_without_permissions(self, enzoic, enzoic_exceptions):
        with pytest.raises(enzoic_exceptions.UnexpectedEnzoicAPIError) as exc_info:
            enzoic(
                os.environ.get("PP_API_KEY_2"), os.environ.get("PP_API_SECRET_2")
            ).get_user_passwords_by_partial_hash("eicar_0@enzoic.com")
        assert "Your account is not allowed to make this call.  Please contact sales@enzoic.com." in str(exc_info.value)


class TestGetDomainExposures:

    def test_get_exposed_users_for_domain_no_details(self, enzoic):
        response = enzoic().get_exposures_for_domain("email.tst", include_exposure_details=False)
        assert response["count"] == 10
        assert len(response["exposures"]) == 10
        assert response["exposures"] == [
            "57ffcf3c1395c80b30dd4429",
            "57dc11964d6db21300991b78",
            "5805029914f33808dc802ff7",
            "598e5b844eb6d82ea07c5783",
            "59bbf691e5017d2dc8a96eab",
            "59bc2016e5017d2dc8bdc36a",
            "59bebae9e5017d2dc85fc2ab",
            "59f36f8c4eb6d85ba0bee09c",
            "5bcf9af3e5017d07201e2149",
            "5c4f818bd3cef70e983dda1e"
        ]

    def test_get_exposed_users_for_domain_with_details(self, enzoic):
        response = enzoic().get_exposures_for_domain("email.tst", include_exposure_details=True)
        assert response["count"] == 10
        assert len(response["exposures"]) == 10
        assert response["exposures"][0] == {
            "id": "57dc11964d6db21300991b78",
            "title": "funsurveys.net",
            "entries": 5123,
            "date": "2015-05-01T00:00:00.000Z",
            "category": "Marketing",
            "passwordType": "Cleartext",
            "exposedData": [
                "Emails",
                "Passwords"
            ],
            "dateAdded": "2016-09-16T15:36:54.000Z",
            "sourceURLs": [],
            "domainsAffected": 683,
            "source": "Unspecified",
            "sourceFileCount": 1
        }

    def test_get_users_for_bad_domain_no_results(self, enzoic):
        response = enzoic().get_exposures_for_domain("@@bogus-domain@@")
        assert response["count"] == 0
        assert len(response["exposures"]) == 0

    def test_get_exposed_users_for_domain_with_specified_page_size(self, enzoic):
        response = enzoic().get_exposures_for_domain("email.tst", include_exposure_details=False, page_size=2)
        assert response["count"] == 10
        assert len(response["exposures"]) == 2
        assert response["pagingToken"] is not None
        assert response["exposures"] == [
            "57ffcf3c1395c80b30dd4429",
            "57dc11964d6db21300991b78",
        ]

    def test_get_exposed_users_for_domain_pagination(self, enzoic):
        domain = "email.tst"
        first_response = enzoic().get_exposures_for_domain(domain, include_exposure_details=False, page_size=2)
        # now retrieve the second page with all the results
        paged_response = enzoic().get_exposures_for_domain(domain, include_exposure_details=True, paging_token=first_response["pagingToken"])
        assert paged_response["count"] == 10
        assert len(paged_response["exposures"]) == 8
        assert paged_response["pagingToken"] is None


class TestAlertSubscriptions:
    TEST_USERS = [
            "eicar_0@enzoic.com",
            "eicar_1@enzoic.com"
        ]

    @pytest.mark.order(1)
    def test_user_alert_subscription_cleanup_previous_test_data(self, enzoic):
        response = enzoic().delete_user_alert_subscriptions(username_hashes=self.TEST_USERS)
        assert response["deleted"] >= 0
        assert response["notFound"] >= 0

    @pytest.mark.order(2)
    def test_add_user_alert_subscription(self, enzoic):
        response = enzoic().add_user_alert_subscriptions(username_hashes=self.TEST_USERS)
        assert response == {
            "added": 2,
            "alreadyExisted": 0,
        }

    @pytest.mark.order(3)
    def test_add_user_alert_subscription_again(self, enzoic):
        response = enzoic().add_user_alert_subscriptions(username_hashes=self.TEST_USERS)
        assert response == {
            "added": 0,
            "alreadyExisted": 2,
        }

    @pytest.mark.order(4)
    def test_user_alert_subscription_cleanup(self, enzoic):
        response = enzoic().delete_user_alert_subscriptions(username_hashes=self.TEST_USERS)
        assert response["deleted"] == 2
        assert response["notFound"] == 0

    @pytest.mark.order(5)
    def test_user_alert_subscription_cleanup_no_data(self, enzoic):
        response = enzoic().delete_user_alert_subscriptions(username_hashes=self.TEST_USERS)
        assert response["deleted"] == 0
        assert response["notFound"] == 2


class TestAlertSubscriptionsCustomData:
    TEST_USERS = [
            "eicar_0@enzoic.com",
            "eicar_1@enzoic.com"
        ]
    CUSTOM_DATA_1 = "123456"
    CUSTOM_DATA_2 = "1234567"

    @pytest.mark.order(6)
    def test_user_alert_subscription_custom_data_cleanup_previous_test_data(self, enzoic):
        response = enzoic().delete_user_alert_subscriptions_with_custom_data(custom_data=self.CUSTOM_DATA_1)
        assert response["deleted"] >= 0
        assert response["notFound"] >= 0

    @pytest.mark.order(7)
    def test_user_alert_subscription_custom_data_cleanup_previous_test_data_2(self, enzoic):
        response = enzoic().delete_user_alert_subscriptions_with_custom_data(custom_data=self.CUSTOM_DATA_2)
        assert response["deleted"] >= 0
        assert response["notFound"] >= 0

    @pytest.mark.order(8)
    def test_add_user_alert_subscription_custom_data(self, enzoic):
        response = enzoic().add_user_alert_subscriptions(username_hashes=self.TEST_USERS, custom_data=self.CUSTOM_DATA_1)
        assert response == {
            "added": 2,
            "alreadyExisted": 0,
        }

    @pytest.mark.order(9)
    def test_add_user_alert_subscription_again_custom_data(self, enzoic):
        response = enzoic().add_user_alert_subscriptions(username_hashes=self.TEST_USERS, custom_data=self.CUSTOM_DATA_1)
        assert response == {
            "added": 0,
            "alreadyExisted": 2,
        }

    @pytest.mark.order(10)
    def test_add_user_alert_subscription_again_different_custom_data(self, enzoic):
        response = enzoic().add_user_alert_subscriptions(username_hashes=self.TEST_USERS, custom_data=self.CUSTOM_DATA_2)
        assert response == {
            "added": 2,
            "alreadyExisted": 0,
        }

    @pytest.mark.order(11)
    def test_user_alert_subscription_cleanup_custom_data(self, enzoic):
        response = enzoic().delete_user_alert_subscriptions_with_custom_data(custom_data=self.CUSTOM_DATA_1)
        assert response["deleted"] == 2
        assert response["notFound"] == 0

    @pytest.mark.order(12)
    def test_user_alert_subscription_cleanup_alt_custom_data(self, enzoic):
        response = enzoic().delete_user_alert_subscriptions_with_custom_data(custom_data=self.CUSTOM_DATA_2)
        assert response["deleted"] == 2
        assert response["notFound"] == 0

    @pytest.mark.order(13)
    def test_user_alert_subscription_cleanup_custom_data_nothing_to_delete(self, enzoic):
        response = enzoic().delete_user_alert_subscriptions_with_custom_data(custom_data=self.CUSTOM_DATA_1)
        assert response["deleted"] == 0
        assert response["notFound"] == 1

    @pytest.mark.order(14)
    def test_user_alert_subscription_cleanup_alt_custom_data_nothing_to_delete(self, enzoic):
        response = enzoic().delete_user_alert_subscriptions_with_custom_data(custom_data=self.CUSTOM_DATA_2)
        assert response["deleted"] == 0
        assert response["notFound"] == 1


class TestGetUserAlertSubscriptions:
    TEST_USER_HASHES = [
        "eicar_0@enzoic.com",
        "eicar_1@enzoic.com",
        "eicar_2@enzoic.com",
        "eicar_3@enzoic.com",
        "eicar_4@enzoic.com",
        "eicar_5@enzoic.com",
        "eicar_6@enzoic.com",
        "eicar_7@enzoic.com",
        "eicar_8@enzoic.com",
        "eicar_9@enzoic.com",
        "eicar_10@enzoic.com",
        "eicar_11@enzoic.com",
        "eicar_12@enzoic.com",
        "eicar_13@enzoic.com",
    ]

    def test_add_large_user_alert_subscriptions(self, enzoic):
        response = enzoic().add_user_alert_subscriptions(username_hashes=self.TEST_USER_HASHES)
        assert response["added"] >= 0
        assert response["alreadyExisted"] >= 0

    def test_user_get_alert_subscriptions(self, enzoic):
        response = enzoic().get_user_alert_subscriptions(4)
        assert response["count"] > 13
        assert len(response["usernameHashes"]) >= 4
        assert response["pagingToken"] is not None


class TestDomainAlertSubscriptions:
    TEST_DOMAINS = [
        "enzoictestadddomaintest1.com",
        "enzoictestadddomaintest2.com"
    ]

    def test_clean_up_previous_add_domain_data(self, enzoic):
        response = enzoic().delete_domain_alert_subscriptions(domains=self.TEST_DOMAINS)
        assert response["deleted"] >= 0
        assert response["notFound"] >= 0

    def test_add_domain_alert_subscriptions(self, enzoic):
        response = enzoic().add_domain_alert_subscriptions(domains=self.TEST_DOMAINS)
        assert response["added"] >= 2
        assert response["alreadyExisted"] >= 0

    def test_add_domain_alert_subscriptions_again(self, enzoic):
        response = enzoic().add_domain_alert_subscriptions(domains=self.TEST_DOMAINS)
        assert response["added"] >= 0
        assert response["alreadyExisted"] >= 2

    def test_delete_domain_alert_subscription(self, enzoic):
        response = enzoic().delete_domain_alert_subscriptions(domains=self.TEST_DOMAINS)
        assert response["deleted"] >= 2
        assert response["notFound"] >= 0

    def test_delete_domain_alert_subscription_again_shows_not_found(self, enzoic):
        response = enzoic().delete_domain_alert_subscriptions(domains=self.TEST_DOMAINS)
        assert response["deleted"] >= 0
        assert response["notFound"] >= 2


class TestIsDomainSubscribedForAlerts:
    TEST_DOMAIN = "enzoictesttestdomaintest1.com"

    def test_add_test_data(self, enzoic):
        response = enzoic().add_domain_alert_subscriptions(self.TEST_DOMAIN)
        assert response["added"] >= 0
        assert response["alreadyExisted"] >= 0

    def test_domain_subscription_exists(self, enzoic):
        response = enzoic().is_domain_subscribed_for_alerts(self.TEST_DOMAIN)
        assert response is True

    def test_delete_test_data(self, enzoic):
        response = enzoic().delete_domain_alert_subscriptions(self.TEST_DOMAIN)
        assert response["deleted"] >= 0
        assert response["notFound"] >= 0

    def test_domain_subscription_doesnt_exist(self, enzoic):
        response = enzoic().is_domain_subscribed_for_alerts(self.TEST_DOMAIN)
        assert response is False


class TestGetDomainAlertSubscriptions:
    TEST_DOMAINS = [
        "enzoictestgetdomaintest1.com",
        "enzoictestgetdomaintest2.com",
        "enzoictestgetdomaintest3.com",
        "enzoictestgetdomaintest4.com",
        "enzoictestgetdomaintest5.com",
        "enzoictestgetdomaintest6.com",
        "enzoictestgetdomaintest7.com",
        "enzoictestgetdomaintest8.com",
        "enzoictestgetdomaintest9.com",
        "enzoictestgetdomaintest10.com",
        "enzoictestgetdomaintest11.com",
        "enzoictestgetdomaintest12.com",
        "enzoictestgetdomaintest13.com",
        "enzoictestgetdomaintest14.com"
    ]

    def test_add_test_data(self, enzoic):
        response = enzoic().add_domain_alert_subscriptions(self.TEST_DOMAINS)
        assert response["added"] >= 0
        assert response["alreadyExisted"] >= 0

    def test_get_subscribed_domains(self, enzoic):
        response = enzoic().get_domain_alert_subscriptions(4)
        assert response["count"] >= 13
        assert len(response["domains"]) == 4
        assert response["pagingToken"] is not None

    def test_get_subscribed_domains_for_next_page(self, enzoic):
        first = enzoic().get_domain_alert_subscriptions(4)
        paged_response = enzoic().get_domain_alert_subscriptions(4, paging_token=first["pagingToken"])
        assert paged_response["count"] >= 13
        assert len(paged_response["domains"]) == 4
        assert paged_response["pagingToken"] is not None





import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="enzoic",
    version="1.40",
    author="Enzoic",
    author_email="mike@enzoic.com",
    description="Python Client for Enzoic",
    url="https://github.com/Enzoic/enzoic-python-client",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    install_requires=["requests", "pytest", "passlib", "bcrypt", "argon2-cffi==23.1.0"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



from enzoic.utilities import hashing


class TestEnzoic:
    def test_unsalted_sha_256(self):
        assert hashing.calc_sha256_unsalted_hash("password") == \
               "5e884898da28047151d0e56f8dc6292773603d0d6aabbdd62a11ef721d1542d8"

    def test_unsalted_sha_1(self):
        assert hashing.calc_sha1_unsalted_hash("password") == "5baa61e4c9b93f3f0682250b6cf8331b7ee68fd8"

    def test_unsalted_md5(self):
        assert hashing.calc_md5_unsalted_hash("password") == "5f4dcc3b5aa765d61d8327deb882cf99"

    def test_ipboard_mybb_hash(self):
        assert hashing.calc_ipboard_mybb_hash("123456", "12345") == "96c06579d8dfc66d81f05aab51a9b284"

    def test_triple_des_hash(self):
        assert hashing.calc_triple_des_hash("qwerty", "yD") == "yDba8kDA7NUDQ"

    def test_vbulletin_pre_385_hash(self):
        assert hashing.calc_vbulletin_pre_3_8_5_hash("123456", "123") == "77d3b7ed9db7d236b9eac8262d27f6a5"

    def test_vbulletin_post_385_hash(self):
        assert hashing.calc_vbulletin_post_3_8_5_hash("123456", "123") == "77d3b7ed9db7d236b9eac8262d27f6a5"

    def test_bcrypt_hash(self):
        assert hashing.calc_bcrypt_hash("12345", "$2a$12$2bULeXwv2H34SXkT1giCZe") == \
               "$2a$12$2bULeXwv2H34SXkT1giCZeJW7A6Q0Yfas09wOCxoIC44fDTYq44Mm"

    def test_crc32_hash(self):
        assert hashing.calc_crc32_hash("password") == "901924565"

    def test_get_phpbb3_hash(self):
        assert hashing.calc_phpbb3_hash("123456789", "$H$993WP3hbz") == "$H$993WP3hbzy0N22X06wxrCc3800D2p41"

    def test_custom_algorithm_1_hash(self):
        assert hashing.calc_custom_algorithm_1_hash("123456", "00new00") == \
               "cee66db36504915f48b2d545803a4494bb1b76b6e9d8ba8c0e6083ff9b281abdef31f6172548fdcde4000e903c5a98a1178c414f7dbf44cffc001aee8e1fe206"

    def test_custom_algorithm_2_hash(self):
        assert hashing.calc_custom_algorithm_2_hash("123456", "123") == \
               "579d9ec9d0c3d687aaa91289ac2854e4"

    def test_sha_512_unsalted_hash(self):
        assert hashing.calc_sha512_unsalted_hash("test") == \
               "ee26b0dd4af7e749aa1a8ee3c10ae9923f618980772e473f8819a5d4940e0db27ac185f8a0e1d5f84f88bc887fd67b143732c304cc5fa9ad8e6f57f50028a8ff"

    def test_custom_algorithm_3_hash(self):
        assert hashing.calc_custom_algorithm_3_hash("test", "123") == "abe45d28281cfa2a4201c9b90a143095"

    def test_md5crypt_hash(self):
        assert hashing.calc_md5crypt_hash("123456", "4d3c09ea") == "$1$4d3c09ea$hPwyka2ToWFbLTOq.yFjf."

    def test_custom_algorithm_4_hash(self):
        assert hashing.calc_custom_algorithm_4_hash(
            "1234", "$2y$12$Yjk3YjIzYWIxNDg0YWMzZOpp/eAMuWCD3UwX1oYgRlC1ci4Al970W"
        ) == "$2y$12$Yjk3YjIzYWIxNDg0YWMzZOpp/eAMuWCD3UwX1oYgRlC1ci4Al970W"

    def test_custom_algorithm_5_hash(self):
        assert hashing.calc_custom_algorithm_5_hash("password", "123456") == \
               "69e7ade919a318d8ecf6fd540bad9f169bce40df4cae4ac1fb6be2c48c514163"

    def test_oscommerce_aef_hash(self):
        assert hashing.calc_oscommerce_aef_hash("password", "123") == "d2bc2f8d09990ebe87c809684fd78c66"

    def test_descrypt_hash(self):
        assert hashing.calc_descrypt_hash("password", "X.") == "X.OPW8uuoq5N."

    def test_get_mysql_pre_4_1(self):
        assert hashing.calc_mysql_pre_4_1("password") == "5d2e19393cc5ef67"

    def test_get_mysql_post_4_1(self):
        assert hashing.calc_mysql_post_4_1("test") == "*94bdcebe19083ce2a1f959fd02f964c7af4cfc29"

    def test_peoplesoft_hash(self):
        assert hashing.calc_peoplesoft_hash("TESTING") == "3weP/BR8RHPLP2459h003IgJxyU="

    def test_punbb_hash(self):
        assert hashing.calc_punbb_hash("password", "123") == "0c9a0dc3dd0b067c016209fd46749c281879069e"

    def test_sha1_salted_hash(self):
        assert hashing.calc_sha1_salted_hash("test", "123") == "7288edd0fc3ffcbe93a0cf06e3568e28521687bc"

    def test_partial_md5_20_hash(self):
        assert hashing.calc_partial_md5_20_hash("password") == "5f4dcc3b5aa765d61d83"

    def test_partial_md5_29_hash(self):
        assert hashing.calc_partial_md5_29_hash("password") == "5f4dcc3b5aa765d61d8327deb882c"

    def test_ave_datalife_diferior_hash(self):
        assert hashing.calc_ave_datalife_diferior_hash("password") == "696d29e0940a4957748fe3fc9efd22a3"

    def test_django_md5_hash(self):
        assert hashing.calc_django_md5_hash("password", "c6218") == "md5$c6218$346abd81f2d88b4517446316222f4276"

    def test_django_sha1_hash(self):
        assert hashing.calc_django_sha1_hash("password", "c6218") == \
               "sha1$c6218$161d1ac8ab38979c5a31cbaba4a67378e7e60845"


    def test_pligg_cms_hash(self):
        assert hashing.calc_pligg_cms_hash("password", "123") == \
               "1230de084f38ace8e3d82597f55cc6ad5d6001568e6"

    def test_run_cms_smf1_1_hash(self):
        assert hashing.calc_run_cms_smf1_1("password", "123") == \
               "0de084f38ace8e3d82597f55cc6ad5d6001568e6"


    def test_ntlm_hash(self):
        assert hashing.calc_ntlm_hash("123456") == "32ed87bdb5fdc5e9cba88547376818d4"

    def test_sha1_dash(self):
        assert hashing.calc_sha1dash_hash("123456", "478c8029d5efddc554bf2fe6bb2219d8c897d4a0") == \
               "55566a759b86fbbd979b579b232f4dd214d08068"

    def test_sha384(self):
        assert hashing.calc_sha384_hash("123456") == \
               "0a989ebc4a77b56a6e2bb7b19d995d185ce44090c13e2984b7ecc6d446d4b61ea9991b76a4c2f04b1b4d244841449454"

    def test_custom_algorithm_7_hash(self):
        assert hashing.calc_custom_algorithm_7_hash("123456", "123456") == \
               "a753d386613efd6d4a534cec97e73890f8ec960fe6634db6dbfb9b2aab207982"

    def test_custom_algorithm_8_hash(self):
        assert hashing.calc_custom_algorithm_8_hash("matthew", "Dn") == \
               "9fc389447b7eb88aff45a1069bf89fbeff89b8fb7d11a6f450583fa4c9c70503"

    def test_custom_algorithm_9_hash(self):
        assert hashing.calc_custom_algorithm_9_hash("0rangepeel", "6kpcxVSjagLgsNCUCr-D") == \
               "07c691fa8b022b52ac1c44cab3e056b344a7945b6eb9db727e3842b28d94fe18c17fe5b47b1b9a29d8149acbd7b3f73866cc12f0a8a8b7ab4ac9470885e052dc"

    def test_sha512crypt_hash_1(self):
        assert hashing.calc_sha512crypt_hash("hashcat", "$6$52450745") == \
               "$6$52450745$k5ka2p8bFuSmoVT1tzOyyuaREkkKBcCNqoDKzYiJL9RaE8yMnPgh2XzzF0NDrUhgrcLwg78xs1w5pJiypEdFX/"

    def test_sha512crypt_hash_2(self):
        assert hashing.calc_sha512crypt_hash("hashcat", "$6$rounds=5000$52450745") == \
               "$6$rounds=5000$52450745$k5ka2p8bFuSmoVT1tzOyyuaREkkKBcCNqoDKzYiJL9RaE8yMnPgh2XzzF0NDrUhgrcLwg78xs1w5pJiypEdFX/"

    def test_sha512crypt_hash_3(self):
        assert hashing.calc_sha512crypt_hash("hashcat", "$6$rounds=4000$52450745") == \
               "$6$rounds=4000$52450745$SpwN1flz4M8T.VckR9l.UofKPTtPvUx3ZfNSAQ.ruUsFBCvC1mz49quqhSrPjK4p25hfLcDZF/86iiA0n38Dh/"


    def test_sha256crypt_hash_1(self):
        assert hashing.calc_sha256crypt_hash("hashcat", "$5$GX7BopJZJxPc/KEK") == \
               "$5$GX7BopJZJxPc/KEK$le16UF8I2Anb.rOrn22AUPWvzUETDGefUmAV8AZkGcD"

    def test_sha256crypt_hash_2(self):
        assert hashing.calc_sha256crypt_hash("hashcat", "$5$rounds=5000$GX7BopJZJxPc/KEK") == \
               "$5$rounds=5000$GX7BopJZJxPc/KEK$le16UF8I2Anb.rOrn22AUPWvzUETDGefUmAV8AZkGcD"

    def test_sha256crypt_hash_3(self):
        assert hashing.calc_sha256crypt_hash("hashcat", "$5$rounds=4000$GX7BopJZJxPc/KEK") == \
               "$5$rounds=4000$GX7BopJZJxPc/KEK$sn.Ds3.Gebi0n6vih/PyOUqlagz5FGk1ITvNh7f1ZMC"

    def test_custom_algorithm_10_hash(self):
        assert hashing.calc_custom_algorithm_10_hash(
            "chatbooks", "NqXCvAHUpAWAco3hVTG5Sg0FfmJRQPKi0LvcHwylzXHhSNuWwvYdMSSGzswi0ZdJ"
        ) == "bd17b9d14010a1d4f8c8077f1be1e20b9364d9979bbcf8591337e952cc6037026aa4a2025543d39169022344b4dd1d20f499395533e35705296034bbf7e7d663"

    def  test_hmac_sha1(self):
        assert hashing.calc_hmac_sha1_salt_as_key("hashcat", "1234") == "d89c92b4400b15c39e462a8caa939ab40c3aeeea"

    def  test_auth_me_sha256(self):
        assert hashing.calc_auth_me_sha256("hashcat", "7218532375810603") == \
               "$SHA$7218532375810603$bfede293ecf6539211a7305ea218b9f3f608953130405cda9eaba6fb6250f824"



class TestCalcArgon2:
    def test_argon2_hash_1(self):
        assert hashing.calc_argon_2_hash("123456", "saltysalt") == \
               "$argon2d$v=19$m=1024,t=3,p=2$c2FsdHlzYWx0$EklGIPtCSWb3IS+q4IQ7rwrwm2o"


    def test_argon2_hash_2(self):
        assert hashing.calc_argon_2_hash("password", "$argon2i$v=19$m=4096,t=2,p=4,l=24$c29tZXNhbHQ") == \
               "$argon2i$v=19$m=4096,t=2,p=4$c29tZXNhbHQ$ZPidoNOWM3jRl0AD+3mGdZsq+GvHprGL"


    def test_argon2_hash_3(self):
        assert hashing.calc_argon_2_hash("123456", "$argon2d$v=19$m=1024,t=3,p=2,l=20$c2FsdHlzYWx0") == \
               "$argon2d$v=19$m=1024,t=3,p=2$c2FsdHlzYWx0$EklGIPtCSWb3IS+q4IQ7rwrwm2o"


    def test_argon2_hash_4(self):
        assert hashing.calc_argon_2_hash("password", "$argon2i$v=19$m=1024,t=2,p=2,l=20$c29tZXNhbHQ") == \
               "$argon2i$v=19$m=1024,t=2,p=2$c29tZXNhbHQ$bBKumUNszaveOgEhcaWl6r6Y91Y"

    def test_argon2_hash_5(self):
        assert hashing.calc_argon_2_hash("password", "$argon2i$v=19$m=4096,t=2,p=4,l=32$c29tZXNhbHQ") == \
               "$argon2i$v=19$m=4096,t=2,p=4$c29tZXNhbHQ$M2X6yo+ZZ8ROwC7MB6/+1yMhGytTzDczBMgo3Is7ptY"


    def test_argon2_hash_6(self):
        assert hashing.calc_argon_2_hash("123456", "$argon2d$v=19$m=10d4,t=ejw,p=2$c2FsdHlzYWx0") == \
               "$argon2d$v=19$m=1024,t=3,p=2$c2FsdHlzYWx0$EklGIPtCSWb3IS+q4IQ7rwrwm2o"
